/*
 * (C) Copyright 2014 Kurento (http://kurento.org/)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */

#ifndef __KMS_ASSEMBLER_H__
#define __KMS_ASSEMBLER_H__

#include <gst/gst.h>
#include "marshal.h"

G_BEGIN_DECLS

typedef struct _KmsAssembler KmsAssembler;

#define KMS_TYPE_ASSEMBLER	(kms_assembler_get_type())
#define KMS_IS_ASSEMBLER(obj)	(GST_IS_MINI_OBJECT_TYPE (obj, KMS_TYPE_ASSEMBLER))
#define KMS_ASSEMBLER_CAST(obj)	((KmsAssembler*)(obj))
#define KMS_ASSEMBLER(obj)	(KMS_ASSEMBLER_CAST(obj))

GType kms_assembler_get_type (void);

/* refcounting */
/**
 * kms_assembler_ref:
 * @a: a #KmsAssembler to increase the refcount of.
 *
 * Increases the refcount of the given marshall buffer by one.
 *
 * Returns: @a
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC KmsAssembler * kms_assembler_ref (KmsAssembler * a);
#endif

static inline KmsAssembler *
kms_assembler_ref (KmsAssembler * a)
{
  return KMS_ASSEMBLER_CAST (gst_mini_object_ref (GST_MINI_OBJECT_CAST (a)));
}

/**
 * kms_assembler_unref:
 * @a: a #KmsAssembler to decrease the refcount of.
 *
 * Decreases the refcount of the assembler. If the refcount reaches 0, the
 * assembler will be freed.
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC void kms_assembler_unref (KmsAssembler * a);
#endif

static inline void
kms_assembler_unref (KmsAssembler * a)
{
  gst_mini_object_unref (GST_MINI_OBJECT_CAST (a));
}

/* copy assembler */
/**
 * kms_assembler_copy:
 * @a: a #KmsAssembler to copy.
 *
 * Copies the given assembler using the copy function of the parent
 *
 * Free-function: kms_assembler_unref
 *
 * Returns: (transfer full): a new copy of @a.
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC KmsAssembler * kms_assembler_copy (const KmsAssembler * a);
#endif

static inline KmsAssembler *
kms_assembler_copy (const KmsAssembler * a)
{
  return KMS_ASSEMBLER_CAST (gst_mini_object_copy (GST_MINI_OBJECT_CONST_CAST (a)));
}

/**
 * kms_assembler_is_writable:
 * @a: a #KmsAssembler
 *
 * Tests if you can safely write data into a assembler.
 */
#define kms_assembler_is_writable(a) \
  gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (a))

/**
 * kms_assembler_make_writable:
 * @a: (transfer full): a #KmsAssembler to make writable
 *
 * Makes a writable buffer from the given assembler.
 *
 * Returns: (transfer full): a new writable assembler
 * (possibly same as @a)
 */
#define kms_assembler_make_writable(a) \
  KMS_ASSEMBLER_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (a)))

/**
 * kms_assembler_replace:
 * @old_assembler: (inout) (transfer full): pointer to a pointer to a
 *     #KmsAssembler to be replaced.
 * @new_assembler: (allow-none) (transfer none): pointer to a
 *     #KmsAssembler that will replace the assembler pointed to by
 *     @old_assembler.
 *
 * Modifies a pointer to a #KmsAssembler to point to a different
 * #KmsAssembler. The modification is done atomically (so this is useful
 * for ensuring thread safety in some cases), and the reference counts are
 * updated appropriately (the old marshall buffer is unreffed, the new one is
 * reffed).
 *
 * Either @new_assembler or the #KmsAssembler pointed to by
 * @old_assembler may be NULL.
 *
 * Returns: TRUE if @new_assembler was different from @old_assembler
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC gboolean kms_assembler_replace (KmsAssembler **old_assembler, KmsAssembler *new_assembler);
#endif

static inline gboolean
kms_assembler_replace (KmsAssembler **old_assembler,
  KmsAssembler *new_assembler)
{
  return gst_mini_object_replace ((GstMiniObject **) old_assembler,
    (GstMiniObject *) new_assembler);
}

/* application specific */

/**
 * kms_assembler_new:
 *
 * Creates a newly allocated assembler without any data.
 *
 * MT safe.
 *
 * Returns: (transfer full): the new #KmsAssembler.
 */
KmsAssembler * kms_assembler_new (KurentoMarshalRules rules);

/**
 * kms_assembler_append_message:
 * @assembler: a #KmsAssembler to append the new message.
 * message: a #KmsMessage to be added (transfer full).
 *
 * Return TRUE is message could be added
 */
gboolean kms_assembler_append_message (KmsAssembler * assembler, KmsMessage *message);

gboolean kms_assembler_is_completed (KmsAssembler * assembler);
void kms_assembler_compose_buffer (KmsAssembler * assembler, gchar **buf, gsize *size);

KmsDataType kms_assembler_get_data_type (const KmsAssembler * assembler);
KurentoMarshalRules kms_assembler_get_encoding_rules (const KmsAssembler * assembler);
G_END_DECLS

#endif /* __KMS_ASSEMBLER_H__ */
