/*
 * (C) Copyright 2014 Kurento (http://kurento.org/)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */

#ifndef __KMS_FRAGMENTER_H__
#define __KMS_FRAGMENTER_H__

#include <gst/gst.h>
#include "marshal.h"

G_BEGIN_DECLS

typedef struct _KmsFragmenter KmsFragmenter;

#define KMS_TYPE_FRAGMENTER	(kms_fragmenter_get_type())
#define KMS_IS_FRAGMENTER(obj)	(GST_IS_MINI_OBJECT_TYPE (obj, KMS_TYPE_FRAGMENTER))
#define KMS_FRAGMENTER_CAST(obj)	((KmsFragmenter*)(obj))
#define KMS_FRAGMENTER(obj)	(KMS_FRAGMENTER_CAST(obj))

GType kms_fragmenter_get_type (void);

/* refcounting */
/**
 * kms_fragmenter_ref:
 * @fragmenter: a #KmsFragmenter to increase the refcount of.
 *
 * Increases the refcount of the given marshall buffer by one.
 *
 * Returns: @fragmenter
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC KmsFragmenter * kms_fragmenter_ref (KmsFragmenter * fragmenter);
#endif

static inline KmsFragmenter *
kms_fragmenter_ref (KmsFragmenter * fragmenter)
{
  return KMS_FRAGMENTER_CAST (gst_mini_object_ref (GST_MINI_OBJECT_CAST (fragmenter)));
}

/**
 * kms_fragmenter_unref:
 * @fragmenter: a #KmsFragmenter to decrease the refcount of.
 *
 * Decreases the refcount of the fragmenter. If the refcount reaches 0, the
 * fragmenter will be freed.
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC void kms_fragmenter_unref (KmsFragmenter * fragmenter);
#endif

static inline void
kms_fragmenter_unref (KmsFragmenter * fragmenter)
{
  gst_mini_object_unref (GST_MINI_OBJECT_CAST (fragmenter));
}

/* copy fragmenter */
/**
 * kms_fragmenter_copy:
 * @fragmenter: a #KmsFragmenter to copy.
 *
 * Copies the given fragmenter using the copy function of the parent
 *
 * Free-function: kms_fragmenter_unref
 *
 * Returns: (transfer full): a new copy of @fragmenter.
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC KmsFragmenter * kms_fragmenter_copy (const KmsFragmenter * fragmenter);
#endif

static inline KmsFragmenter *
kms_fragmenter_copy (const KmsFragmenter * fragmenter)
{
  return KMS_FRAGMENTER_CAST (gst_mini_object_copy (GST_MINI_OBJECT_CONST_CAST (fragmenter)));
}

/**
 * kms_fragmenter_is_writable:
 * @fragmenter: a #KmsFragmenter
 *
 * Tests if you can safely write data into a fragmenter.
 */
#define kms_fragmenter_is_writable(fragmenter) \
  gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (fragmenter))

/**
 * kms_fragmenter_make_writable:
 * @fragmenter: (transfer full): a #KmsFragmenter to make writable
 *
 * Makes a writable buffer from the given fragmenter.
 *
 * Returns: (transfer full): a new writable fragmenter
 * (possibly same as @fragmenter)
 */
#define kms_fragmenter_make_writable(fragmenter) \
  KMS_FRAGMENTER_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (fragmenter)))

/**
 * kms_fragmenter_replace:
 * @old_fragmenter: (inout) (transfer full): pointer to a pointer to a
 *     #KmsFragmenter to be replaced.
 * @new_fragmenter: (allow-none) (transfer none): pointer to a
 *     #KmsFragmenter that will replace the fragmenter pointed to by
 *     @old_fragmenter.
 *
 * Modifies a pointer to a #KmsFragmenter to point to a different
 * #KmsFragmenter. The modification is done atomically (so this is useful
 * for ensuring thread safety in some cases), and the reference counts are
 * updated appropriately (the old marshall buffer is unreffed, the new one is
 * reffed).
 *
 * Either @new_fragmenter or the #KmsFragmenter pointed to by
 * @old_fragmenter may be NULL.
 *
 * Returns: TRUE if @new_fragmenter was different from @old_fragmenter
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC gboolean kms_fragmenter_replace (KmsFragmenter **old_fragmenter, KmsFragmenter *new_fragmenter);
#endif

static inline gboolean
kms_fragmenter_replace (KmsFragmenter **old_fragmenter,
  KmsFragmenter *new_fragmenter)
{
  return gst_mini_object_replace ((GstMiniObject **) old_fragmenter,
    (GstMiniObject *) new_fragmenter);
}

/* application specific */

/**
 * kms_fragmenter_new:
 *
 * Creates a newly allocated fragmenter without any data.
 *
 * MT safe.
 *
 * Returns: (transfer full): the new #KmsFragmenter.
 */
KmsFragmenter * kms_fragmenter_new (KurentoMarshalRules rules, gsize max);

void kms_fragmenter_set_message_type (KmsFragmenter *fragmenter, KmsMessageType type);

gboolean kms_fragmenter_event (KmsFragmenter *fragmenter, guint id, const GstEvent *event, GError **err);
gboolean kms_fragmenter_query (KmsFragmenter *fragmenter, guint id, const GstQuery *query, GError **err);
gboolean kms_fragmenter_buffer (KmsFragmenter *fragmenter, guint id, const GstBuffer *buffer, GError **err);

guint kms_fragmenter_n_messages (const KmsFragmenter *fragmenter);

/**
 * kms_fragmenter_nth_message:
 *
 * Gets the message in the position n.
 *
 * Returns: (transfer none): the #KmsMessage.
 */
const KmsMessage * kms_fragmenter_nth_message (const KmsFragmenter *fragmenter, guint n);

void kms_fragmenter_for_each_message (const KmsFragmenter *fragmenter, GFunc func, gpointer user_data);

guint kms_fragmenter_coded_data_size (const KmsFragmenter *fragmenter);

KurentoMarshalRules kms_fragmenter_get_encoding_rules (const KmsFragmenter *fragmenter);

gboolean kms_fragmenter_is_serialized (const KmsFragmenter *fragmenter);

G_END_DECLS

#endif /* __KMS_FRAGMENTER_H__ */