/*
 * (C) Copyright 2014 Kurento (http://kurento.org/)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */

#ifndef __KMS_MESSAGE_H__
#define __KMS_MESSAGE_H__

#include <gst/gst.h>

G_BEGIN_DECLS

typedef struct _KmsMessage KmsMessage;

#define KMS_TYPE_MESSAGE	(kms_message_get_type())
#define KMS_IS_MESSAGE(obj)	(GST_IS_MINI_OBJECT_TYPE (obj, KMS_TYPE_MESSAGE))
#define KMS_MESSAGE_CAST(obj)	((KmsMessage*)(obj))
#define KMS_MESSAGE(obj)	(KMS_MESSAGE_CAST(obj))

typedef enum {
  KMS_DATA_TYPE_UNKNOWN,
  KMS_DATA_TYPE_QUERY,
  KMS_DATA_TYPE_EVENT,
  KMS_DATA_TYPE_BUFFER
} KmsDataType;

typedef enum {
  GST_MARSHALL_REQUEST,
  GST_MARSHALL_RESPONSE
} KmsMessageType;

GType kms_message_get_type (void);

/* refcounting */
/**
 * kms_message_ref:
 * @m: a #KmsMessage to increase the refcount of.
 *
 * Increases the refcount of the given marshall buffer by one.
 *
 * Returns: @m
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC KmsMessage * kms_message_ref (KmsMessage * m);
#endif

static inline KmsMessage *
kms_message_ref (KmsMessage * m)
{
  return KMS_MESSAGE_CAST (gst_mini_object_ref (GST_MINI_OBJECT_CAST (m)));
}

/**
 * kms_message_unref:
 * @m: a #KmsMessage to decrease the refcount of.
 *
 * Decreases the refcount of the message. If the refcount reaches 0, the
 * message will be freed.
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC void kms_message_unref (KmsMessage * m);
#endif

static inline void
kms_message_unref (KmsMessage * m)
{
  gst_mini_object_unref (GST_MINI_OBJECT_CAST (m));
}

/* copy message */
/**
 * kms_message_copy:
 * @m: a #KmsMessage to copy.
 *
 * Copies the given message using the copy function of the parent
 *
 * Free-function: kms_message_unref
 *
 * Returns: (transfer full): a new copy of @m.
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC KmsMessage * kms_message_copy (const KmsMessage * m);
#endif

static inline KmsMessage *
kms_message_copy (const KmsMessage * m)
{
  return KMS_MESSAGE_CAST (gst_mini_object_copy (GST_MINI_OBJECT_CONST_CAST (m)));
}

/**
 * kms_message_is_writable:
 * @m: a #KmsMessage
 *
 * Tests if you can safely write data into a message.
 */
#define kms_message_is_writable(m) \
  gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (m))

/**
 * kms_message_make_writable:
 * @m: (transfer full): a #KmsMessage to make writable
 *
 * Makes a writable buffer from the given message.
 *
 * Returns: (transfer full): a new writable message
 * (possibly same as @m)
 */
#define kms_message_make_writable(m) \
  KMS_MESSAGE_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (m)))

/**
 * kms_message_replace:
 * @old_message: (inout) (transfer full): pointer to a pointer to a
 *     #KmsMessage to be replaced.
 * @new_message: (allow-none) (transfer none): pointer to a
 *     #KmsMessage that will replace the message pointed to by
 *     @old_message.
 *
 * Modifies a pointer to a #KmsMessage to point to a different
 * #KmsMessage. The modification is done atomically (so this is useful
 * for ensuring thread safety in some cases), and the reference counts are
 * updated appropriately (the old marshall buffer is unreffed, the new one is
 * reffed).
 *
 * Either @new_message or the #KmsMessage pointed to by
 * @old_message may be NULL.
 *
 * Returns: TRUE if @new_message was different from @old_message
 */
#ifdef _FOOL_GTK_DOC_
G_INLINE_FUNC gboolean kms_message_replace (KmsMessage **old_message, KmsMessage *new_message);
#endif

static inline gboolean
kms_message_replace (KmsMessage **old_message,
  KmsMessage *new_message)
{
  return gst_mini_object_replace ((GstMiniObject **) old_message,
    (GstMiniObject *) new_message);
}

/* application specific */

/**
 * kms_message_new:
 *
 * Creates a newly allocated message without any data.
 *
 * MT safe.
 *
 * Returns: (transfer full): the new #KmsMessage.
 */
KmsMessage * kms_message_new (guint id);
guint kms_message_get_req_id (const KmsMessage *msg);

void kms_message_set_type_counter (KmsMessage * msg, guint32 n);
guint32 kms_message_get_type_counter (const KmsMessage *msg);

void kms_message_set_fragment_id (KmsMessage * msg, guint id);
guint kms_message_get_fragment_id (const KmsMessage *msg);

void kms_message_set_total_fragments (KmsMessage * msg, guint n);
guint kms_message_get_total_fragments (const KmsMessage *msg);

void kms_message_set_message_type (KmsMessage * msg, KmsMessageType type);
KmsMessageType kms_message_get_message_type (const KmsMessage *msg);

KmsDataType kms_message_get_message_data_type (const KmsMessage *msg);

void kms_message_set_data (KmsMessage * msg, KmsDataType type, gchar * buf, gsize size, gboolean own);
void kms_message_get_data (const KmsMessage *msg, KmsDataType *type, const gchar ** buf, gsize * size);

gboolean kms_message_is_equal (const KmsMessage *msg1, const KmsMessage *msg2);

G_END_DECLS

#endif /* __KMS_MESSAGE_H__ */